from oelint_adv.cls_rule import Rule
from oelint_adv.cls_item import *
from oelint_adv.const_func import FUNC_ORDER
import os


class TaskCustomOrder(Rule):
    def __init__(self):
        super().__init__(id="oelint.task.customorder",
                         severity="error",
                         message="<FOO>")

    def check(self, _file, stash):
        res = []
        items = stash.GetItemsFor(
            filename=_file, classifier=TaskAdd.CLASSIFIER)
        for item in items:
            _before = item.Before
            _after = item.After
            for _a in _after:
                if not _a in FUNC_ORDER:
                    continue
                _ai = FUNC_ORDER.index(_a)
                for _b in _before:
                    if not _b in FUNC_ORDER:
                        continue
                    _bi = FUNC_ORDER.index(_b)
                    if _ai > _bi:
                        self.OverrideMsg(
                            "after '{}' and before '{}' breaks ordering cylce".format(_a, _b))
                        res += self.finding(item.Origin, item.InFileLine)
        return res



import os
import glob
from oelint_adv.cls_item import *


class Rule():
    def __init__(self, id="", severity="", message=""):
        self.ID = id
        self.Severity = severity
        self.Msg = message

    def check(self, _file, stash):
        return []

    def fix(self, _file, stash):
        return []

    def finding(self, _file, _line, override_msg=None):
        return ["{}:{}:{}:{}:{}".format(os.path.abspath(_file), _line, self.Severity, self.ID, override_msg or self.Msg)]

    def __repr__(self):
        return "{}".format(self.ID)

    def FormatMsg(self, *args):
        return self.Msg.format(*args)

    def OverrideMsg(self, newmsg):
        self.Msg = newmsg


def load_rules():
    res = []
    for file in glob.glob(os.path.join(os.path.dirname(os.path.abspath(__file__)), "rule_*.py")):
        name = os.path.splitext(os.path.basename(file))[0]
        for m in ["oelint_adv." + name]:  # , "." + name, name]:
            try:
                module = __import__(name)
                for member in dir(module):
                    try:
                        if issubclass(getattr(module, member), Rule):
                            inst = getattr(module, member)()
                            if inst.ID:
                                res.append(inst)
                    except:
                        pass
                break
            except Exception as e:
                pass
    return res



import os
import glob
from oelint_adv.cls_stash import Stash
from oelint_adv.cls_item import Comment, Function, Include, Item, PythonBlock, Variable


def get_files(stash, _file, pattern):
    res = []
    src_uris = stash.GetItemsFor(filename=_file, classifier=Variable.CLASSIFIER,
                                 attribute=Variable.ATTR_VAR, attributeValue="SRC_URI")
    files_paths = list(
        set(["{}/*/{}".format(os.path.dirname(x.Origin), pattern) for x in src_uris]))
    for item in src_uris:
        files_paths += list(set(["{}/*/{}".format(os.path.dirname(x.Origin), pattern)
                                 for x in stash.GetItemsFor(filename=item.Origin)]))
    for item in files_paths:
        res += glob.glob(item)
    return list(set(res))


def get_scr_components(string):
    comp = string.replace("\t", "").strip(
        " \n\"").replace("://", ";", 1).split(";")
    if len(comp) < 2:
        return {"proto": "dummy", "name": ""}
    res = {"proto": comp[0], "name": comp[1]}
    for i in range(2, len(comp)):
        sp = comp[i].split("=")
        if len(sp) > 1:
            res[sp[0]] = sp[1]
    return res



from oelint_adv.cls_rule import Rule
from oelint_adv.cls_item import *
from oelint_adv.helper_files import get_files
from urllib.parse import urlparse
import os


class FilePatchIsUpstreamStatus(Rule):
    def __init__(self):
        super().__init__(id="oelint.file.upstreamstatus",
                         severity="info",
                         message="Patch '{FILE}' should should contain a Upstream-Status entry")

    def check(self, _file, stash):
        res = []
        items = get_files(stash, _file, "*.patch")
        _items = stash.GetItemsFor(filename=_file, classifier=Variable.CLASSIFIER,
                                   attribute=Variable.ATTR_VAR, attributeValue="SRC_URI")
        for i in items:
            with open(i) as _input:
                content = _input.readline()
                if not any([x for x in content if x.startswith("Upstream-Status: ")]):
                    # Find matching SRC_URI assignment
                    _assign = [x for x in _items if x.VarValue.find(
                        os.path.basename(i)) != -1]
                    if any(_assign):
                        self.OverrideMsg(self.Msg.replace(
                            "{FILE}", os.path.basename(i)))
                        res += self.finding(_assign[0].Origin,
                                            _assign[0].InFileLine)
        return res



from oelint_adv.cls_rule import Rule
import re


class NoSpaceTrailingRule(Rule):
    def __init__(self):
        super().__init__(id="oelint.spaces.lineend",
                         severity="warning",
                         message="Line shall not end with a space")

    def __getMatches(self, _file, stash):
        res = []
        items = stash.GetItemsFor(filename=_file)
        for i in items:
            _linecnt = 0
            for line in i.Raw.split("\n"):
                if line.endswith("  "):
                    res.append(i)
                _linecnt += 1
        return res

    def check(self, _file, stash):
        res = []
        for i in self.__getMatches(_file, stash):
            res += self.finding(i.Origin, i.InFileLine + _linecnt)
        return res

    def fix(self, _file, stash):
        res = []
        for i in self.__getMatches(_file, stash):
            i.Raw = re.sub(r"\s{2,}\n", "\n", i.Raw)
            res.append(_file)
        return res



from oelint_adv.cls_rule import Rule
from oelint_adv.cls_item import *
import re


class VarMultiInherit(Rule):
    def __init__(self):
        super().__init__(id="oelint.var.multiinherit",
                         severity="warning",
                         message="'{INH}' is included multiple times")

    def check(self, _file, stash):
        res = []
        items = stash.GetItemsFor(filename=_file, classifier=Variable.CLASSIFIER,
                                  attribute=Variable.ATTR_VAR, attributeValue="inherit")
        keys = []
        for i in items:
            keys += [x.strip() for x in re.split("\s|,", i.VarValue) if x]
        for key in list(set(keys)):
            _i = [x for x in items if x.VarValue.find(key) != -1]
            if len(_i) > 1:
                res += self.finding(_i[-1].Origin, _i[-1].InFileLine,
                                    self.Msg.replace("{INH}", key))
        return res
